@@ -24,6 +24,22 @@ struct config_file {
     int fd;
 };
 
+enum section {
+    SECTION_MAIN,
+    SECTION_LOW,
+    SECTION_NORMAL,
+    SECTION_CRITICAL,
+    SECTION_COUNT,
+};
+
+static const char *const section_names[] = {
+    [SECTION_MAIN] = "main",
+    [SECTION_LOW] = "low",
+    [SECTION_NORMAL] = "normal",
+    [SECTION_CRITICAL] = "critical",
+};
+
+
 static const char *
 get_user_home_dir(void)
 {
@@ -271,6 +287,16 @@ parse_section_urgency(const char *key, const char *value,
             layer = ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM;
         else if (strcasecmp(value, "overlay") == 0)
             layer = ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY;
+        else if (strcmp(key, "progress-style") == 0) {
+            if (strcasecmp(value, "bar") == 0)
+                conf->progress.style = PROGRESS_STYLE_BAR;
+            else if (strcasecmp(value, "background") == 0)
+                conf->progress.style = PROGRESS_STYLE_BACKGROUND;
+            else {
+                LOG_ERR("%s:%d: invalid progress style: %s", path, lineno, value);
+                return false;
+            }
+        }
         else {
             LOG_ERR(
                 "%s:%u: %s: invalid layer value, must be one of "
@@ -394,7 +420,12 @@ parse_section_urgency(const char *key, const char *value,
         conf->body.format = ambstoc32(value);
     }
 
-    else if (strcmp(key, "progress-bar-color") == 0) {
+    else if (strcmp(key, "progress-color") == 0 || strcmp(key, "progress-bar-color") == 0) {
+        if (strcmp(key, "progress-bar-color") == 0) {
+            LOG_WARN("%s:%d: 'progress-bar-color' is deprecated, use 'progress-color' instead",
+                    path, lineno);
+        }
+
         pixman_color_t color;
         if (!str_to_color(value, &color, path, lineno))
             return false;
@@ -837,7 +868,12 @@ parse_section_main(const char *key, const char *value, struct config *conf,
         }
     }
 
-    else if (strcmp(key, "progress-bar-color") == 0) {
+    else if (strcmp(key, "progress-color") == 0 || strcmp(key, "progress-bar-color") == 0) {
+        if (strcmp(key, "progress-bar-color") == 0) {
+            LOG_WARN("%s:%d: 'progress-bar-color' is deprecated, use 'progress-color' instead",
+                    path, lineno);
+        }
+
         pixman_color_t color;
         if (!str_to_color(value, &color, path, lineno))
             return false;
@@ -863,6 +899,21 @@ parse_section_main(const char *key, const char *value, struct config *conf,
         }
     }
 
+    else if (strcmp(key, "progress-style") == 0) {
+        enum progress_style style;
+        if (strcasecmp(value, "bar") == 0)
+            style = PROGRESS_STYLE_BAR;
+        else if (strcasecmp(value, "background") == 0)
+            style = PROGRESS_STYLE_BACKGROUND;
+        else {
+            LOG_ERR("%s:%d: invalid progress style: %s", path, lineno, value);
+            return false;
+        }
+        
+        for (int i = 0; i < 3; i++)
+            conf->by_urgency[i].progress.style = style;
+    }
+
     else if (strcmp(key, "max-timeout") == 0) {
         unsigned long max_timeout_secs;
         if (!str_to_ulong(value, 10, &max_timeout_secs)) {
@@ -929,15 +980,7 @@ parse_section_main(const char *key, const char *value, struct config *conf,
 }
 
 static bool
-parse_config_file(FILE *f, struct config *conf, const char *path)
-{
-    enum section {
-        SECTION_MAIN,
-        SECTION_LOW,
-        SECTION_NORMAL,
-        SECTION_CRITICAL,
-        SECTION_COUNT,
-    } section = SECTION_MAIN;
+parse_config_file_section(FILE *f, struct config *conf, const char *path, enum section target_sec) {
 
     /* Function pointer, called for each key/value line */
     typedef bool (*parser_fun_t)(
@@ -952,16 +995,14 @@ parse_config_file(FILE *f, struct config *conf, const char *path)
         [SECTION_CRITICAL] = &parse_section_critical,
     };
 
-    static const char *const section_names[] = {
-        [SECTION_MAIN] = "main",
-        [SECTION_LOW] = "low",
-        [SECTION_NORMAL] = "normal",
-        [SECTION_CRITICAL] = "critical",
-    };
-
     unsigned lineno = 0;
     char *_line = NULL;
     size_t count = 0;
+    /* If target_sec is main we can parse as if we are inside the main section */
+    bool inside_target_sec = target_sec == SECTION_MAIN;
+
+    parser_fun_t section_parser = section_parser_map[target_sec];
+    assert(section_parser != NULL);
 
     while (true) {
         errno = 0;
@@ -1008,19 +1049,28 @@ parse_config_file(FILE *f, struct config *conf, const char *path)
 
             *end = '\0';
 
-            for (section = SECTION_MAIN; section < SECTION_COUNT; ++section) {
-                if (strcmp(&key_value[1], section_names[section]) == 0) {
-                    break;
+            bool invalid_sec = true;
+            for (enum section s = SECTION_MAIN; s < SECTION_COUNT; ++s) {
+                if (strcmp(&key_value[1], section_names[s]) == 0) {
+                    invalid_sec = false;
+                    if (s == target_sec) {
+                        inside_target_sec = true;
+                    }else {
+                        inside_target_sec = false;
+                    }
                 };
             }
 
-            if (section == SECTION_COUNT) {
+            if (invalid_sec) {
                 LOG_ERR("%s:%d: invalid section name: %s", path, lineno, &key_value[1]);
                 goto err;
             }
 
             continue;
         }
+        if (!inside_target_sec) {
+            continue;
+        }
 
         char *key = strtok(key_value, "=");
         if (key == NULL) {
@@ -1060,9 +1110,6 @@ parse_config_file(FILE *f, struct config *conf, const char *path)
         LOG_DBG("section=%s, key='%s', value='%s'",
                 section_names[section], key, value);
 
-        parser_fun_t section_parser = section_parser_map[section];
-        assert(section_parser != NULL);
-
         if (!section_parser(key, value, conf, path, lineno))
             goto err;
     }
@@ -1075,6 +1122,18 @@ err:
     return false;
 }
 
+static bool
+parse_config_file(FILE *f, struct config *conf, const char *path)
+{
+    for (enum section sec=SECTION_MAIN; sec < SECTION_COUNT; sec++) {
+        rewind(f);
+        if (!parse_config_file_section(f, conf, path, sec)){
+            return false;
+        }
+    }
+    return true;
+}
+
 bool
 config_load(struct config *conf, const char *path)
 {
@@ -1086,7 +1145,7 @@ config_load(struct config *conf, const char *path)
         .max_width = 0,
         .max_height = 0,
         .dpi_aware = false,
-        .icon_theme_name = strdup("hicolor"),
+        .icon_theme_name = strdup("default"),
         .max_icon_size = 48,
         .stacking_order = STACK_BOTTOM_UP,
         .anchor = ANCHOR_TOP_RIGHT,
@@ -1126,6 +1185,7 @@ config_load(struct config *conf, const char *path)
                 .progress = {
                     .height = 20,
                     .color = {0xffff, 0xffff, 0xffff, 0xffff},
+                    .style = PROGRESS_STYLE_BAR,
                 },
                 .max_timeout_secs = 0,
                 .default_timeout_secs = 0,
