@@ -4,8 +4,11 @@
 enum ctrl_command {
     CTRL_QUIT,
     CTRL_LIST,
+    CTRL_PAUSE,
+    CTRL_UNPAUSE,
     CTRL_DISMISS_BY_ID, CTRL_DISMISS_ALL,
     CTRL_ACTIONS_BY_ID,
+    CTRL_DISMISS_WITH_DEFAULT_ACTION_BY_ID,
 };
 
 struct ctrl_request {
